#include "header.h"

// todo: DESTCOLORKEY

/*
'"Note that the DDBLTFX_NOTEARING, DDBLTFX_MIRRORLEFTRIGHT, and DDBLTFX_MIRRORUPDOWN flags are unsupported 
on Windows 2000 and later and are never passed to the driver."' 
http://msdn.microsoft.com/en-us/library/windows/hardware/ff550474%28v=vs.85%29.aspx

... so here we do mirroring by hand ...
*/


HRESULT BltProc(  
	LPDIRECTDRAWSURFACE7 lpDDDestSurface,
	LPRECT lpDestRect,                    
	LPDIRECTDRAWSURFACE7 lpDDSrcSurface,    
	LPRECT lpSrcRect,                    
	DWORD dwFlags,                          
	LPDDBLTFX lpDDBltFx )
{
	// todo: Mirroring
	return lpDDDestSurface->Blt( lpDestRect, lpDDSrcSurface, lpSrcRect, dwFlags, lpDDBltFx ); 
}

HRESULT BltProc(  
	LPDIRECTDRAWSURFACE4 lpDDDestSurface,
	LPRECT lpDestRect,                    
	LPDIRECTDRAWSURFACE4 lpDDSrcSurface,    
	LPRECT lpSrcRect,                    
	DWORD dwFlags,                          
	LPDDBLTFX lpDDBltFx )
{
	// todo: Mirroring
	return lpDDDestSurface->Blt( lpDestRect, lpDDSrcSurface, lpSrcRect, dwFlags, lpDDBltFx ); 
}

HRESULT BltProc(  
	LPDIRECTDRAWSURFACE3 lpDDDestSurface,
	LPRECT lpDestRect,                    
	LPDIRECTDRAWSURFACE3 lpDDSrcSurface,    
	LPRECT lpSrcRect,                    
	DWORD dwFlags,                          
	LPDDBLTFX lpDDBltFx )
{
	return BltProc(
		(LPDIRECTDRAWSURFACE) lpDDDestSurface,
		lpDestRect,
		(LPDIRECTDRAWSURFACE) lpDDSrcSurface,    
		lpSrcRect,                    
		dwFlags,                          
		lpDDBltFx
	);
}

HRESULT BltProc(  
	LPDIRECTDRAWSURFACE2 lpDDDestSurface,
	LPRECT lpDestRect,                    
	LPDIRECTDRAWSURFACE2 lpDDSrcSurface,    
	LPRECT lpSrcRect,                    
	DWORD dwFlags,                          
	LPDDBLTFX lpDDBltFx )
{
	return BltProc(
		(LPDIRECTDRAWSURFACE) lpDDDestSurface,
		lpDestRect,
		(LPDIRECTDRAWSURFACE) lpDDSrcSurface,    
		lpSrcRect,                    
		dwFlags,                          
		lpDDBltFx
	);
}

HRESULT BltProc(  
	LPDIRECTDRAWSURFACE lpDDDestSurface,
	LPRECT lpDestRect,                    
	LPDIRECTDRAWSURFACE lpDDSrcSurface,    
	LPRECT lpSrcRect,                    
	DWORD dwFlags,                          
	LPDDBLTFX lpDDBltFx )
{
	BOOL IsMirror = FALSE;
	HRESULT hResult;
	IDirectDrawSurface* lpDDWorkSurface = NULL;
		if( Config.BltMirror && 
		lpDDSrcSurface != NULL &&
		dwFlags & DDBLT_DDFX &&
		lpDDBltFx->dwDDFX & (DDBLTFX_MIRRORLEFTRIGHT | DDBLTFX_MIRRORUPDOWN)  ) 
	{
		DDSURFACEDESC ddsd;		
		IDirectDraw* ddraw;
		DWORD dwX;
		DWORD dwY;
		DWORD dwWidth;
		DWORD dwHeight;
		if( GetHighestDDInterface( &ddraw ) )
		{ 
			if( lpSrcRect == NULL )
			{
				ddsd.dwSize = sizeof(ddsd);
				hResult = lpDDSrcSurface->GetSurfaceDesc(&ddsd);
				if( FAILED( hResult ) ) return hResult;
				dwX = 0;
				dwY = 0;
				dwWidth = ddsd.dwWidth;					
				dwHeight= ddsd.dwHeight;		
			} else {
				dwX = lpSrcRect->left;
				dwY = lpSrcRect->top;
				dwWidth = lpSrcRect->right - lpSrcRect->left;
				dwHeight = lpSrcRect->bottom - lpSrcRect->top;
			}
			RtlSecureZeroMemory(&ddsd, sizeof(ddsd));
			ddsd.dwSize = sizeof(ddsd);
			ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
			ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;
			ddsd.dwWidth = dwWidth;
			ddsd.dwHeight = dwHeight;
			if( SUCCEEDED( ddraw->CreateSurface( &ddsd, &lpDDWorkSurface, NULL ) ) )
			{
				if( dwFlags & DDBLT_KEYSRC )
				{
					DDCOLORKEY ddck;
					if( SUCCEEDED( lpDDSrcSurface->GetColorKey( DDCKEY_SRCBLT, &ddck ) ) )
					{
						lpDDWorkSurface->SetColorKey( DDCKEY_SRCBLT, &ddck );
					}
				}
				HDC hdcWork; 
				if( SUCCEEDED( lpDDWorkSurface->GetDC( &hdcWork ) ) )
				{
					HDC hdcSrc;
					if( SUCCEEDED( lpDDSrcSurface->GetDC( &hdcSrc ) ) )
					{	
						if( StretchBlt( 
							hdcWork, 0, 0, ddsd.dwWidth, ddsd.dwHeight,
							hdcSrc, 
							( lpDDBltFx->dwDDFX & DDBLTFX_MIRRORLEFTRIGHT ) ? (dwX + (dwWidth - 1)) : dwX,
							( lpDDBltFx->dwDDFX & DDBLTFX_MIRRORUPDOWN ) ? (dwY + (dwHeight - 1)) : dwY,
							( lpDDBltFx->dwDDFX & DDBLTFX_MIRRORLEFTRIGHT ) ? (~(dwWidth - 1)) : dwWidth,
							( lpDDBltFx->dwDDFX & DDBLTFX_MIRRORUPDOWN ) ? (~(dwHeight - 1)) : dwHeight,
							SRCCOPY ) )
						{
							lpDDBltFx->dwDDFX &= (~(DDBLTFX_MIRRORLEFTRIGHT | DDBLTFX_MIRRORUPDOWN));
							lpSrcRect = NULL;
							IsMirror = TRUE;
						}
						lpDDSrcSurface->ReleaseDC( hdcSrc );	
					}
					lpDDWorkSurface->ReleaseDC( hdcWork );	
				}
			}
		}
	}

	if( IsMirror ) lpDDSrcSurface = lpDDWorkSurface;

	hResult = lpDDDestSurface->Blt( lpDestRect, lpDDSrcSurface, lpSrcRect, dwFlags, lpDDBltFx );
	if(	lpDDWorkSurface ) lpDDWorkSurface->Release();
	return hResult;
}








